
#include <allegro.h>
#include <stdio.h>
#include "../melee.h"

ShipData::ShipData(char *filename) :
  data(NULL),
  bitmapPanel(NULL),
  spriteShip(NULL),
  spriteWeapon(NULL),
  spriteWeaponExplosion(NULL),
  spriteSpecial(NULL),
  spriteSpecialExplosion(NULL),
  spriteExtra(NULL),
  spriteExtraExplosion(NULL),
  sampleWeapon(NULL),
  sampleSpecial(NULL),
  sampleExtra(NULL),
  moduleVictory(NULL)
{
  int i, index = 0, count, rotate;
  char *str;

  data = load_datafile(filename);
  if(!data) {
    allegro_exit();
    printf("Error loading %s\n", filename);
    exit(1);
  }

  set_config_data((char *)(data[index].dat), data[index].size);
  index++;

  count = get_config_int("Objects", "PanelBitmaps", 0);
  if(count > 0) {
    bitmapPanel = new (BITMAP *)[count];
    for(i = 0; i < count; i++) {
      bitmapPanel[i] = (BITMAP *)(data[index].dat);
      index++;
    }
  }


  rotate = FALSE;
  str = get_config_string("Objects", "ShipSprites", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteShip = new SpaceSprite(&data[index], count);
    else
      spriteShip = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  rotate = FALSE;
  str = get_config_string("Objects", "WeaponSprites", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteWeapon = new SpaceSprite(&data[index], count);
    else
      spriteWeapon = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  rotate = FALSE;
  str = get_config_string("Objects", "WeaponExplosion", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteWeaponExplosion = new SpaceSprite(&data[index], count);
    else
      spriteWeaponExplosion = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }


  rotate = FALSE;
  str = get_config_string("Objects", "SpecialSprites", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteSpecial = new SpaceSprite(&data[index], count);
    else
      spriteSpecial = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  rotate = FALSE;
  str = get_config_string("Objects", "SpecialExplosion", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteSpecialExplosion = new SpaceSprite(&data[index], count);
    else
      spriteSpecialExplosion = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  rotate = FALSE;
  str = get_config_string("Objects", "ExtraSprites", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteExtra = new SpaceSprite(&data[index], count);
    else
      spriteExtra = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  rotate = FALSE;
  str = get_config_string("Objects", "ExtraExplosion", "0");
  if(str[strlen(str) - 1] == 'r') {
    str[strlen(str) - 1] = '\0';
    rotate = TRUE;
  }
  count = atoi(str);
  if(count > 0) {
    if(!rotate)
      spriteExtraExplosion = new SpaceSprite(&data[index], count);
    else
      spriteExtraExplosion = new SpaceRotatingSprite(&data[index], count);
    index += count;
  }

  moduleVictory = (JGMOD *)(data[index].dat);
  index++;

  count = get_config_int("Objects", "WeaponSamples", 0);
  if(count > 0) {
    sampleWeapon = new (SAMPLE *)[count];
    for(i = 0; i < count; i++) {
      sampleWeapon[i] = (SAMPLE *)(data[index].dat);
      index++;
    }
  }

  count = get_config_int("Objects", "SpecialSamples", 0);
  if(count > 0) {
    sampleSpecial = new (SAMPLE *)[count];
    for(i = 0; i < count; i++) {
      sampleSpecial[i] = (SAMPLE *)(data[index].dat);
      index++;
    }
  }

  count = get_config_int("Objects", "ExtraSamples", 0);
  if(count > 0) {
    sampleExtra = new (SAMPLE *)[count];
    for(i = 0; i < count; i++) {
      sampleExtra[i] = (SAMPLE *)(data[index].dat);
      index++;
    }
  }
}

ShipData::~ShipData()
{
  if(spriteShip)
    delete spriteShip;
  if(spriteWeapon)
    delete spriteWeapon;
  if(spriteWeaponExplosion)
    delete spriteWeaponExplosion;
  if(spriteSpecial)
    delete spriteSpecial;
  if(spriteSpecialExplosion)
    delete spriteSpecialExplosion;
  if(spriteExtra)
    delete spriteExtra;

  unload_datafile(data);
}