
#include <math.h>
#include "../melee.h"

#define MAX_DOGIS 4

class ChenjesuBroodhome : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          shardDamage;
  int          shardArmour;
  int          weaponDamage;
  int          weaponArmour;
  int          weaponFired;
  Shot        *weaponObject;

  double       specialVelocity;
  int          specialFuelSap;
  int          specialArmour;
  double       specialAccelRate;
  double       specialMass;
  int          specialNumDOGIs;

  public:
  ChenjesuBroodhome(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
};

class ChenjesuShot : public AnimatedShot {
  public:
  ChenjesuShot(double ox, double oy, double oangle, double ov, int odamage,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void inflict_damage(SpaceObject *other);
};

class ChenjesuDOGI : public AnimatedShot {
  int    sap_factor;
  int    fuel_flag;
  double accel_rate;
  int   *num_dogis;

  public:
  ChenjesuDOGI(double ox, double oy, double oangle, double ov, int fuel_sap,
    int oarmour, double oaccel, double omass, Ship *oship,
    SpaceSprite *osprite, int *onum_dogis);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage();
  virtual void animateExplosion();
  virtual void soundExplosion();
};

ChenjesuBroodhome::ChenjesuBroodhome(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  shardDamage    = get_config_int("Weapon", "ShardDamage", 0);
  shardArmour    = get_config_int("Weapon", "ShardArmour", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponFired    = FALSE;
  weaponObject   = NULL;

  specialVelocity    = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialFuelSap     = get_config_int("Special", "FuelSap", 0);
  specialArmour      = get_config_int("Special", "Armour", 0);
  specialAccelRate   = (double)(get_config_int("Special", "AccelRate", 0)) / ACCELRATE_RATIO;
  specialMass        = (double)(get_config_int("Special", "Mass", 0));
  specialNumDOGIs    = 0;
}

int ChenjesuBroodhome::activate_weapon()
{
  if(weaponFired)
    return(FALSE);
  weaponObject = new ChenjesuShot(0.0, -(height() / 2.0), angle,
    weaponVelocity, weaponDamage, weaponArmour, this,
    data->spriteWeapon, 64, 5);
  spaceFrame->addSpaceObject(LAYER_SHOTS, weaponObject);
  weaponFired = TRUE;
  return(TRUE);
}

int ChenjesuBroodhome::activate_special()
{
  if(specialNumDOGIs >= MAX_DOGIS)
    return(FALSE);

  spaceFrame->addSpaceObject(LAYER_SHOTS, new ChenjesuDOGI(0.0, height() / 1.5,
    angle + 180.0, specialVelocity, specialFuelSap, specialArmour,
    specialAccelRate, specialMass, this, data->spriteSpecial,
    &specialNumDOGIs));
  return(TRUE);
}

void ChenjesuBroodhome::calculate()
{
  int i;

  Ship::calculate();

  if((weaponObject != NULL) && (!weaponObject->exists()))
    weaponObject = NULL;
  if(weaponFired && (!fire_weapon))
    weaponFired = FALSE;
  if((weaponObject != NULL) && (!fire_weapon)) {
    weaponObject->stop();
    for(i = 0; i < 8; i++)
      spaceFrame->addSpaceObject(LAYER_SHOTS, new AnimatedShot(0.0, 0.0,
        45.0 * (double)(i), weaponVelocity, shardDamage, weaponRange,
        shardArmour, this, weaponObject, data->spriteExtra, 64, 5));
    weaponObject->destroy();
    weaponObject = NULL;
    stop_sample(data->sampleExtra[0]);
    play_sample(data->sampleExtra[0], 255, 128, 1000, FALSE);
  }
}

ChenjesuShot::ChenjesuShot(double ox, double oy, double oangle, double ov,
  int odamage, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, -1.0, oarmour, oship, oship,
    osprite, ofcount, ofsize)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 31;
  explosionFrameSize  = 5;
}

void ChenjesuShot::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other);
  if (other->mass > 0) state = 0;
  if(other->getID() == SPACE_SHIP)
    spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new FixedAnimation(other,
      explosionSprite, explosionFrameCount, explosionFrameSize));
  else
    spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(x, y,
      explosionSprite, explosionFrameCount, explosionFrameSize));
}

ChenjesuDOGI::ChenjesuDOGI(double ox, double oy, double ov, double oangle,
  int fuel_sap, int oarmour, double accel, double omass, Ship *oship,
  SpaceSprite *osprite, int *onum_dogis) :
  AnimatedShot(ox, oy, ov, oangle, 0, -1.0, oarmour, oship, oship,
    osprite, 64, 5),
  sap_factor(fuel_sap),
  accel_rate(accel),
  num_dogis(onum_dogis)
{
  mass = omass;
  fuel_flag = collide_flag;
  collide_flag = 0;
  (*num_dogis)++;
}

void ChenjesuDOGI::calculate()
{
  double vel;

  AnimatedShot::calculate();

  if((ship->target) && (!ship->target->isAlly(ship))) {
    angle = trajectory_angle(ship->target);

    if((fabs(ship->target->get_angle() - (trajectory_angle(ship->target) + 180.0)) < 22.5) ||
      (fabs(ship->target->get_angle() - (trajectory_angle(ship->target) - 180.0)) < 22.5))
      angle += 180.0;
    while(angle > 360.0)
      angle -= 360.0;
    while(angle < 0.0)
      angle += 360.0;
  }

  vx += cos(angle * ANGLE_RATIO) * accel_rate;
  vy += sin(angle * ANGLE_RATIO) * accel_rate;

  vel = sqrt((vx * vx) + (vy * vy));

  if(vel > v) {
    vx = vx * v / vel;
    vy = vy * v / vel;
  }

  driftvx *= 0.99;
  driftvy *= 0.99;

  if(state == 0)
    (*num_dogis)--;
}

void ChenjesuDOGI::inflict_damage(SpaceObject *other)
{
  if(other->collideFlag() == 0)
    return;

  stop_sample(ship->data->sampleExtra[1]);
  play_sample(ship->data->sampleExtra[1], 255, 128, 1000, FALSE);

  if(other->collideFlag() == fuel_flag)
    return;

  other->fuel_sap += sap_factor;
  other->handle_damage();
}

void ChenjesuDOGI::handle_damage()
{
  AnimatedShot::handle_damage();
  if(state == 0)
    (*num_dogis)--;
}

void ChenjesuDOGI::animateExplosion()
{
  spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(x, y,
    kaboomSprite, KABOOM_FRAMES, 5));
}

void ChenjesuDOGI::soundExplosion()
{
  stop_sample(ship->data->sampleExtra[2]);
  play_sample(ship->data->sampleExtra[2], 255, 128, 1000, FALSE);
}

Ship *getChenjesuBroodhome(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new ChenjesuBroodhome(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO ChenjesuBroodhomeInfo =
{
  "chebr",
  "Chenjesu Broodhome",
  getChenjesuBroodhome
};
