
#include <math.h>
#include "../melee.h"

class DruugeMauler : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  double       weaponDriftVelocity;

  public:
  DruugeMauler(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  protected:
  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

class DruugeMissile : public Missile {
  public:
  DruugeMissile(double ox, double oy, double oangle, double ov, int odamage,
      double weaponDriftVelocity, double orange, int oarmour, Ship *oship,
      SpaceSprite *osprite);
  virtual void inflict_damage (SpaceObject *other);
  double kick;
};

DruugeMauler::DruugeMauler(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData) {
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange         = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity      = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage        = get_config_int("Weapon", "Damage", 0);
  weaponArmour        = get_config_int("Weapon", "Armour", 0);
  weaponDriftVelocity = (double)(get_config_int("Weapon", "DriftVelocity", 0)) / VELOCITY_RATIO;
}

int DruugeMauler::activate_weapon() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new DruugeMissile(
      0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage,
      weaponDriftVelocity, weaponRange, weaponArmour, this,
      data->spriteWeapon));
  vx -= (cos(angle * ANGLE_RATIO) * weaponDriftVelocity);
  vy -= (sin(angle * ANGLE_RATIO) * weaponDriftVelocity);

  return(TRUE);
}

void DruugeMauler::calculate_fire_special() {
  if((fire_special) && (crew > 1) && (batt < batt_max) &&
    (special_recharge == 0)) {

    batt += special_drain;
    if(batt > batt_max)
      batt = batt_max;
    crew--;

    special_recharge = special_rate;

    stop_sample(data->sampleSpecial[0]);
    play_sample(data->sampleSpecial[0], 255, 128, 1000, FALSE);
  }
}

DruugeMissile::DruugeMissile(double ox, double oy, double oangle, double ov,
    int odamage, double weaponDriftVelocity, double orange, int oarmour,
    Ship *oship, SpaceSprite *osprite) :
    Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
    kick(weaponDriftVelocity)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 5;
  explosionFrameSize  = 5;
}
void DruugeMissile::inflict_damage (SpaceObject *other) {
  other->pull(this, -kick);
  Missile::inflict_damage(other);
}

Ship *getDruugeMauler(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new DruugeMauler(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO DruugeMaulerInfo = {
  "druma",
  "Druuge Mauler",
  getDruugeMauler
};
