
#include <math.h>
#include "../melee.h"

class KohrAhBlade;

class KohrAhMarauder : public Ship
{
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;
  int          weaponFired;
  KohrAhBlade *weaponObject;

  double       specialVelocity;
  int          specialDamage;
  int          specialArmour;
  double       specialRange;

  public:
  KohrAhMarauder(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate();
};

class KohrAhBlade : public AnimatedShot {
  int       passive;
  double    passiveRange;

  public:
  KohrAhBlade(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
    int ofcount, int ofsize);

  virtual void calculate();
  virtual void animateExplosion();

  void disengage();
};

class KohrAhBladeDecay : public Animation {
  public:
  KohrAhBladeDecay(double ox, double oy, double ovx, double ovy,
    SpaceSprite *osprite, int ofcount, int ofsize);
};

class KohrAhFRIED : public Shot {
  int frame_count;

  public:
  KohrAhFRIED(double oangle, double ov, int odamage, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount);

  virtual void calculate();
};

KohrAhMarauder::KohrAhMarauder(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
  weaponObject   = NULL;
  weaponFired    = FALSE;

  specialRange    = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialVelocity = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialArmour   = get_config_int("Special", "Armour", 0);
}

int KohrAhMarauder::activate_weapon()
{
  if(weaponFired)
    return(FALSE);
  weaponObject = new KohrAhBlade(0.0, -(h / 2.0), angle, weaponVelocity,
    weaponDamage, weaponRange, weaponArmour, this, data->spriteWeapon, 10, 4);
  spaceFrame->addSpaceObject(LAYER_SHOTS, weaponObject);
  weaponFired = TRUE;
  return(TRUE);
}

int KohrAhMarauder::activate_special()
{
  int i;

  for(i = 0; i < 16; i++)
    spaceFrame->addSpaceObject(LAYER_SHOTS, new KohrAhFRIED(
      ((double)(i) * 22.5) - 180.0, specialVelocity, specialDamage,
      specialRange, specialArmour, this, data->spriteSpecial, 20));

  return(TRUE);
}

void KohrAhMarauder::calculate()
{
  Ship::calculate();
  if((weaponFired) && (!fire_weapon)) {
    weaponObject->disengage();
    weaponFired = FALSE;
    weaponObject = NULL;
  }
}

KohrAhBlade::KohrAhBlade(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount, int ofsize) :
  AnimatedShot(ox, oy, oangle, ov, odamage, -1.0, oarmour, oship, oship,
    osprite, ofcount, ofsize),
  passive(FALSE),
  passiveRange(orange)
{
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 4;
}

void KohrAhBlade::calculate()
{
  int locked = FALSE, i;
  SpaceObject *o;

  AnimatedShot::calculate();

  if(passive) {
    for(i = 0; (i < SPACE_LAYERS) && (!locked); i++) {
      o = spaceFrame->getFirstObject(i);
      while((o != NULL) && (!locked)) {
        if((!o->isAlly(this)) && (o->collideFlag() >= 0) &&
          (distance(o) < passiveRange)) {
          angle = trajectory_angle(o);
          vx = (v / 10.0) * (cos(angle * ANGLE_RATIO));
          vy = (v / 10.0) * (sin(angle * ANGLE_RATIO));
          locked = TRUE;
        }
        o = o->next;
      }
    }

    if((!locked) && (ship) && (ship->target) && (!ship->target->isAlly(ship))) {
      angle = trajectory_angle(ship->target);
      vx = (v / 10.0) * (cos(angle * ANGLE_RATIO));
      vy = (v / 10.0) * (sin(angle * ANGLE_RATIO));
      locked = TRUE;
    }
    if(!locked) {
      vx = 0.0;
      vy = 0.0;
    }
  }
}

void KohrAhBlade::animateExplosion()
{
  spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new KohrAhBladeDecay(x, y,
    vx, vy, explosionSprite, explosionFrameCount, explosionFrameSize));
}

void KohrAhBlade::disengage()
{
  passive = TRUE;
}

KohrAhBladeDecay::KohrAhBladeDecay(double ox, double oy, double ovx,
  double ovy, SpaceSprite *osprite, int ofcount, int ofsize) :
  Animation(ox, oy, osprite, ofcount, ofsize)
{
  vx = ovx;
  vy = ovy;
}

KohrAhFRIED::KohrAhFRIED(double oangle, double ov, int odamage,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofcount) :
  Shot(0.0, 0.0, oangle, ov, odamage, orange, oarmour, oship, oship, osprite),
  frame_count(ofcount)
{
  x += cos(angle * ANGLE_RATIO) * 50.0;
  y += sin(angle * ANGLE_RATIO) * 50.0;
}

void KohrAhFRIED::calculate() {
  sprite_index = (int)((d / range) * (double)(frame_count));
  Shot::calculate();
}

Ship *getKohrAhMarauder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new KohrAhMarauder(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO KohrAhMarauderInfo =
{
  "kohma",
  "Kohr-Ah Marauder",
  getKohrAhMarauder
};
