
#include "../melee.h"

#define T_FORM 0
#define Y_FORM 1

class MmrnmhrmXForm : public Ship {
  struct {
    public:
    double       speed_max;
    double       accel_rate;
    double       turn_rate;
    int          recharge_amount;
    int          recharge_rate;
    int          weapon_drain;
    int          weapon_rate;
    int          hotspot_rate;
    SpaceSprite *sprite;
  } form_data[2];

  int form;

  int        laserColor;
  double     laserRange;
  int        laserFrames;
  int        laserDamage;
  SpaceLine *laser1;
  SpaceLine *laser2;
  int        laserSpark;

  double missileRange;
  double missileVelocity;
  int    missileDamage;
  int    missileArmour;
  double missileTurnRate;

  public:
  MmrnmhrmXForm(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  void calculate();
  void handle_damage();

  int activate_weapon();
  int activate_special();
};

MmrnmhrmXForm::MmrnmhrmXForm(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData),
  laser1(NULL),
  laser2(NULL),
  laserSpark(0)
{
  form_data[T_FORM].speed_max       = speed_max;
  form_data[T_FORM].accel_rate      = accel_rate;
  form_data[T_FORM].turn_rate       = turn_rate;
  form_data[T_FORM].recharge_amount = recharge_amount;
  form_data[T_FORM].recharge_rate   = recharge_rate;
  form_data[T_FORM].weapon_drain    = weapon_drain;
  form_data[T_FORM].weapon_rate     = weapon_rate;
  form_data[T_FORM].hotspot_rate    = hotspot_rate;
  form_data[T_FORM].sprite          = sprite;

  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  laserColor  = get_config_int("Weapon1", "Color", 0);
  laserRange  = (double)(get_config_int("Weapon1", "Range", 0)) * RANGE_RATIO;
  laserFrames = get_config_int("Weapon1", "Frames", 0);
  laserDamage = get_config_int("Weapon1", "Damage", 0);

  missileRange    = (double)(get_config_int("Weapon2", "Range", 0)) * RANGE_RATIO;
  missileVelocity = (double)(get_config_int("Weapon2", "Velocity", 0)) / VELOCITY_RATIO;
  missileDamage   = get_config_int("Weapon2", "Damage", 0);
  missileArmour   = get_config_int("Weapon2", "Armour", 0);
  missileTurnRate = (double)(get_config_int("Weapon2", "TurnRate", 0));

  weapon_drain    = get_config_int("Special", "WeaponDrain", 0);
  weapon_rate     = get_config_int("Special", "WeaponRate", 0);
  hotspot_rate    = get_config_int("Special", "HotspotRate", 0);
  recharge_amount = get_config_int("Special", "RechargeAmount", 0);
  recharge_rate   = get_config_int("Special", "RechargeRate", 0);
  turn_rate       = (double)(get_config_int("Special", "TurnRate", 0));
  speed_max       = (double)(get_config_int("Special", "SpeedMax", 0));
  accel_rate      = (double)(get_config_int("Special", "AccelRate", 0));

  calculate_speed_max();
  calculate_accel_rate();
  calculate_turn_rate();
  scale_frame_rate(&weapon_rate);
  scale_frame_rate(&recharge_rate);
  scale_frame_rate(&hotspot_rate);

  form_data[Y_FORM].speed_max       = speed_max;
  form_data[Y_FORM].accel_rate      = accel_rate;
  form_data[Y_FORM].turn_rate       = turn_rate;
  form_data[Y_FORM].recharge_amount = recharge_amount;
  form_data[Y_FORM].recharge_rate   = recharge_rate;
  form_data[Y_FORM].weapon_drain    = weapon_drain;
  form_data[Y_FORM].weapon_rate     = weapon_rate;
  form_data[Y_FORM].hotspot_rate    = hotspot_rate;
  form_data[Y_FORM].sprite          = data->spriteSpecial;

  form = Y_FORM;
  activate_special();
}

void MmrnmhrmXForm::calculate()
{
  if(laserSpark > 0) {
    laserSpark--;
    if(laserSpark == 0) {
      if((laser1->get_length() == laserRange) &&
        (laser2->get_length() == laserRange))
        spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(
          laser1->normal_x() + laser1->edge_x(),
          laser1->normal_y() + laser1->edge_y(),
          sparkSprite, SPARK_FRAMES, 5));
      laser1 = NULL;
      laser2 = NULL;
    }
  }

  Ship::calculate();
}

int MmrnmhrmXForm::activate_weapon()
{
  if(form == T_FORM) {
    laser1 = spaceFrame->addSpaceLine(new Laser(angle + 4.01,
      pallete_color[laserColor], laserRange, laserDamage, laserFrames, this,
      -28.0, -2.0));
    laser2 = spaceFrame->addSpaceLine(new Laser(angle - 4.01,
      pallete_color[laserColor], laserRange, laserDamage, laserFrames, this,
      28.0, -2.0));
    laserSpark = laserFrames;
  }
  if(form == Y_FORM) {
    spaceFrame->addSpaceObject(LAYER_SHOTS, new HomingMissile(-13.0, -2.0,
      angle - 25.0, missileVelocity, missileDamage, missileRange,
      missileArmour, missileTurnRate, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new HomingMissile(13.0, -2.0,
      angle + 25.0, missileVelocity, missileDamage, missileRange,
      missileArmour, missileTurnRate, this, data->spriteWeapon));
  }
  return(TRUE);
}

int MmrnmhrmXForm::activate_special()
{
  form = ((!form) & 1);
  speed_max       = form_data[form].speed_max;
  accel_rate      = form_data[form].accel_rate;
  turn_rate       = form_data[form].turn_rate;
  recharge_amount = form_data[form].recharge_amount;
  recharge_rate   = form_data[form].recharge_rate;
  weapon_drain    = form_data[form].weapon_drain;
  weapon_rate     = form_data[form].weapon_rate;
  hotspot_rate    = form_data[form].hotspot_rate;
  sprite          = form_data[form].sprite;
  weapon_sample   = form;
  return(TRUE);
}

void MmrnmhrmXForm::handle_damage()
{
  if(speed_loss > 0.0) {
    form_data[T_FORM].speed_max *= (1.0 - (1.0 / (double)(speed_loss)));
    form_data[T_FORM].turn_rate /= (1.0 + (1.0 / (double)(speed_loss)));
    form_data[T_FORM].hotspot_rate++;
    form_data[Y_FORM].speed_max *= (1.0 - (1.0 / (double)(speed_loss)));
    form_data[Y_FORM].turn_rate /= (1.0 + (1.0 / (double)(speed_loss)));
    form_data[Y_FORM].hotspot_rate++;
  }

  Ship::handle_damage();
}

Ship *getMmrnmhrmXForm(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new MmrnmhrmXForm(sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO MmrnmhrmXFormInfo =
{
  "mmrxf",
  "Mmrnmhrm X-Form",
  getMmrnmhrmXForm
};
