
#include "../melee.h"

class ShofixtiScout : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialRange;
  int          specialFrames;
  int          specialDamage;
  int          flipSwitch;
  int          glory;

  public:
  ShofixtiScout(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual void calculate_fire_special();
};

ShofixtiScout::ShofixtiScout(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponDamage   = get_config_int("Weapon", "Armour", 0);

  specialRange  = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialFrames = get_config_int("Special", "Frames", 0);
  specialDamage = get_config_int("Special", "Damage", 0);
  flipSwitch    = FALSE;
  glory         = 0;
}

int ShofixtiScout::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(0.0, -(h / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

void ShofixtiScout::calculate_fire_special()
{
  SpaceObject *o;
  double minDist;
  int gloryDamage, i;

  if(fire_special) {
    if(!flipSwitch) {
      flipSwitch = TRUE;
      glory++;
      if(glory == 3) {
        state = 0;
        for(i = 0; i < SPACE_LAYERS; i++) {
          o = spaceFrame->getFirstObject(i);
          while(o != NULL) {
            minDist = specialRange + (width() / 2.0) + (o->width() / 2.0);
            if((!o->isAlly(this)) && (o->collideFlag() >= 0) &&
              (distance(o) <= minDist)) {
              gloryDamage = (int)((minDist - distance(o)) /
                specialRange * (double)(specialDamage));
              if(gloryDamage > specialDamage)
                gloryDamage = specialDamage;
              o->damage += gloryDamage;
              o->handle_damage();
            }
            o = o->next;
          }
        }
        if((target) && (target->exists()))
          meleeVictory = target->data->moduleVictory;
        else
          meleeVictory = data->moduleVictory;
        spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(x, y,
          data->spriteSpecial, specialFrames, 5));
        stop_sample(data->sampleSpecial[0]);
        play_sample(data->sampleSpecial[0], 255, 128, 1000, FALSE);
      } else {
        blit(data->bitmapPanel[5 + glory], panelBitmap[1], 0, 23, 0, 23,
          CAPTAIN_WIDTH, 7);
        blit(data->bitmapPanel[5 + glory], panelBitmap[2], 0, 23, 0, 23,
          CAPTAIN_WIDTH, 7);
        blit(data->bitmapPanel[5 + glory], panelBitmap[3], 0, 23, 0, 23,
          CAPTAIN_WIDTH, 7);
        blit(data->bitmapPanel[5 + glory], panelBitmap[4], 0, 23, 0, 23,
          CAPTAIN_WIDTH, 7);
        blit(data->bitmapPanel[5 + glory], panelBitmap[5], 0, 23, 0, 23,
          CAPTAIN_WIDTH, 7);
      }
    }
  } else
    flipSwitch = FALSE;
}

Ship *getShofixtiScout(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new ShofixtiScout(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO ShofixtiScoutInfo =
{
  "shosc",
  "Shofixti Scout",
  getShofixtiScout
};
