
#include <math.h>
#include "../melee.h"

class SyreenPenetrator : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialRange;
  double       specialVelocity;
  int          specialDamage;
  int          specialFrames;

  public:
  SyreenPenetrator(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
};

class CrewPod : public SpaceObject {
  Ship *ship;
  ShipData *shipData;

  int frame_count;
  int frame_size;
  int frame_step;

  double velocity;
  int    life;
  int    lifetime;

  public:
  CrewPod(double ox, double oy, double oVelocity, int oLifeTime,
    Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();

  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage();
};

SyreenPenetrator::SyreenPenetrator(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialRange    = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialVelocity = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialDamage   = get_config_int("Special", "Damage", 0);
  specialFrames   = get_config_int("Special", "Frames", 0);
}

int SyreenPenetrator::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(0.0, -(height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

int SyreenPenetrator::activate_special()
{
  if((!target) || (!target->exists()) || (target->getCrew() < 2))
    return(FALSE);

  int callDamage, i;
  double minDist;

  minDist = specialRange + (width() / 2.0) + (target->width() / 2.0);
  if(distance(target) <= minDist) {
    callDamage = (int)((minDist - distance(target)) /
      specialRange * (double)(specialDamage));
    if(callDamage > specialDamage)
      callDamage = specialDamage;
    callDamage += (random() % specialDamage);
    if((target->getCrew() - callDamage) < 1)
      callDamage = target->getCrew() - 1;
    target->directDamage += callDamage;
    target->handle_damage();
    for(i = 0; i < callDamage; i++) {
      spaceFrame->addSpaceObject(LAYER_SHOTS, new CrewPod(
        target->normal_x() + (cos((trajectory_angle(target) - 180.0) *
          ANGLE_RATIO) * target->width()) + (double)(random() % 50) - 25.0,
        target->normal_y() + (sin((trajectory_angle(target) - 180.0) *
          ANGLE_RATIO) * target->height()) + (double)(random() % 50) - 25.0,
        specialVelocity, specialFrames, this, data->spriteSpecial, 32, 5));
    }
  }

  return(TRUE);
}

CrewPod::CrewPod(double ox, double oy, double oVelocity, int oLifeTime,
  Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize) :
  SpaceObject(ox, oy, 0.0, 0, osprite),
  ship(oship),
  shipData(oship->data),
  frame_count(ofcount),
  frame_size(ofsize),
  frame_step(0),
  velocity(oVelocity),
  life(0),
  lifetime(oLifeTime)
{
  ally_flag = ship->target->collideFlag();
  if(!ship->exists())
    state = 0;
}

void CrewPod::calculate()
{
  frame_step++;
  if(frame_step == frame_size) {
    frame_step = 0;
    sprite_index++;
    if(sprite_index == frame_count)
      sprite_index = 0;
  }

  life++;
  if(life == lifetime) {
    state = 0;
    return;
  }

  if(!ship->exists())
    state = 0;
  else {
    vx = cos(trajectory_angle(ship) * ANGLE_RATIO) * velocity;
    vy = sin(trajectory_angle(ship) * ANGLE_RATIO) * velocity;
  }

  SpaceObject::calculate();
}

void CrewPod::inflict_damage(SpaceObject *other)
{
  if(other->getID() == SPACE_SHIP) {
    stop_sample(shipData->sampleExtra[0]);
    play_sample(shipData->sampleExtra[0], 255, 128, 1000, FALSE);
    ((Ship *)(other))->repair++;
    other->handle_damage();
    state = 0;
  }
}

void CrewPod::handle_damage()
{
  state = 0;
  damage = 0;
  directDamage = 0;
}

Ship *getSyreenPenetrator(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new SyreenPenetrator(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO SyreenPenetratorInfo =
{
  "syrpe",
  "Syreen Penetrator",
  getSyreenPenetrator
};
