
#include <math.h>
#include "../melee.h"

class ThraddashTorch : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  double       specialThrust;
  int          specialDamage;

  public:
  ThraddashTorch(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
  virtual void calculate_thrust();
};

class ThraddashFlame : public Animation {
  Ship *ship;

  public:
  ThraddashFlame(double ox, double oy, int odamage, Ship *oship,
    SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
  virtual void handle_damage();
};

ThraddashTorch::ThraddashTorch(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialThrust = (double)(get_config_int("Special", "Thrust", 0)) / VELOCITY_RATIO;
  specialDamage = get_config_int("Special", "Damage", 0);
}

int ThraddashTorch::activate_weapon() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(0.0, -(height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteWeapon));
  return(TRUE);
}

int ThraddashTorch::activate_special() {
  spaceFrame->addSpaceObject(LAYER_SHOTS, new ThraddashFlame(
    x + cos((angle - 180.0) * ANGLE_RATIO) * (width() / 2.0),
    y + sin((angle - 180.0) * ANGLE_RATIO) * (height() / 2.0),
    specialDamage, this, data->spriteSpecial, 40, 10));
  driftvx += cos(angle * ANGLE_RATIO) * specialThrust;
  driftvy += sin(angle * ANGLE_RATIO) * specialThrust;
  return(TRUE);
}

void ThraddashTorch::calculate_thrust()
{
  if(special_recharge == 0)
    Ship::calculate_thrust();
}

ThraddashFlame::ThraddashFlame(double ox, double oy, int odamage,
  Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize) :
  Animation(ox, oy, osprite, ofcount, ofsize),
  ship(oship)
{
  id = SPACE_SHOT;
  collide_flag = ship->collideFlag();
  ally_flag = collide_flag;
  damage_factor = odamage;

  if(!ship->exists())
    state = 0;
}

void ThraddashFlame::calculate()
{
  if(!ship->exists())
    state = 0;
  else
    Animation::calculate();
}

void ThraddashFlame::inflict_damage(SpaceObject *other)
{
  SpaceObject::inflict_damage(other); 
  state = 0;
  spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(normal_x(),
    normal_y(), sparkSprite, SPARK_FRAMES, 5));
}

void ThraddashFlame::handle_damage()
{
  if((damage > 0) || (directDamage > 0)) {
    state = 0;
    spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(normal_x(),
      normal_y(), sparkSprite, SPARK_FRAMES, 5));
  }
}

Ship *getThraddashTorch(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new ThraddashTorch(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO ThraddashTorchInfo =
{
  "thrto",
  "Thraddash Torch",
  getThraddashTorch
};
