
#include "../melee.h"

class UtwigJugger : public Ship {
  double       weaponRange;
  double       weaponVelocity;
  int          weaponDamage;
  int          weaponArmour;

  public:
  UtwigJugger(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  protected:
  virtual void calculate_fire_weapon();
  virtual void animate(Frame *space, Frame *radar);
  virtual void handle_damage();
};

UtwigJugger::UtwigJugger(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange    = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage   = get_config_int("Weapon", "Damage", 0);
  weaponArmour   = get_config_int("Weapon", "Armour", 0);
}

void UtwigJugger::calculate_fire_weapon()
{
  weapon_low = FALSE;

  if(fire_weapon) {
    if(weapon_recharge != 0)
      return;

    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(34.0, -11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(-34.0, -11.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(18.0, -20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(-18.0, -20.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(6.0, -27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));
    spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(-6.0, -27.0,
      angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
      this, this, data->spriteWeapon));

    weapon_recharge = weapon_rate;

    stop_sample(data->sampleWeapon[0]);
    play_sample(data->sampleWeapon[0], 255, 128, 1000, FALSE);
  }
}

void UtwigJugger::animate(Frame *space, Frame *radar)
{
  if(phase_count < PHASE_MAX)
    return;
  if(special_recharge != 0)
    sprite->animate_character((int)(corner_x()), (int)(corner_y()), sprite_index, pallete_color[hot_color[special_recharge % HOT_COLORS]], space);
  else
    sprite->animate((int)(corner_x()), (int)(corner_y()), sprite_index, space);
  if(radar_color >= 0) {
    putpixel(radar->frame,
      (int)(normal_x() / (X_MAX / 56.0)),
      (int)(normal_y() / (Y_MAX / 48.0)), pallete_color[radar_color]);
    radar->add_pixel(
      (int)(normal_x() / (X_MAX / 56.0)),
      (int)(normal_y() / (Y_MAX / 48.0)), 1, 1);
  }
}

void UtwigJugger::handle_damage()
{
  if(special_recharge != 0) {
    batt += damage;
    if(batt > batt_max)
      batt = batt_max;
    damage = 0;
  }
  Ship::handle_damage();
}

Ship *getUtwigJugger(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new UtwigJugger(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO UtwigJuggerInfo =
{
  "utwju",
  "Utwig Jugger",
  getUtwigJugger
};
