
#include <math.h>
#include "../melee.h"

class VuxIntruder : public Ship {
  int    weaponColor;
  double weaponRange;
  int    weaponFrames;
  int    weaponDamage;

  double specialRange;
  double specialVelocity;
  double specialSlowdown;
  int    specialArmour;

  public:
  VuxIntruder(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual int activate_special();
};

class VuxLimpet : public AnimatedShot {
  double slowdown_factor;

  public:
  VuxLimpet(double ox, double oy, double ov, double slowdown, double orange,
    int oarmour, Ship *oship, SpaceSprite *osprite, int ofcount, int ofsize);

  virtual void calculate();
  virtual void inflict_damage(SpaceObject *other);
};

VuxIntruder::VuxIntruder(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData)
{
  weaponColor  = get_config_int("Weapon", "Color", 0);
  weaponRange  = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponFrames = get_config_int("Weapon", "Frames", 0);
  weaponDamage = get_config_int("Weapon", "Damage", 0);

  specialRange    = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialVelocity = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialSlowdown = (double)(get_config_int("Special", "Slowdown", 0)) / VELOCITY_RATIO;
  specialArmour   = get_config_int("Special", "Armour", 0);
}

int VuxIntruder::activate_weapon()
{
  spaceFrame->addSpaceLine(new Laser(angle,
    pallete_color[weaponColor], weaponRange, weaponDamage, weaponFrames,
    this, 0.0, -(h / 2.0)));
  return(TRUE);
}

int VuxIntruder::activate_special()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new VuxLimpet(0.0, height() / 2.0,
    specialVelocity, specialSlowdown, specialRange, specialArmour, this,
    data->spriteSpecial, 11, 5));
  return(TRUE);
}

VuxLimpet::VuxLimpet(double ox, double oy, double ov, double slowdown,
  double orange, int oarmour, Ship *oship, SpaceSprite *osprite,
  int ofsize, int ofcount) :
  AnimatedShot(ox, oy, 0.0, ov, 0, orange, oarmour, oship, oship, osprite,
    ofcount, ofsize),
  slowdown_factor(slowdown)
{
  if((ship->target) && (!ship->target->isAlly(ship)))
    angle = trajectory_angle(ship->target);
  else
    angle = ship->get_angle() - 180.0;

  vx    = v * cos(angle * ANGLE_RATIO);
  vy    = v * sin(angle * ANGLE_RATIO);

  if(!ship->exists())
    state = 0;
}

void VuxLimpet::calculate()
{
  if(!ship->exists()) {
    state = 0;
    return;
  }

  if((ship->target) && (!ship->target->isAlly(ship)))
    angle = trajectory_angle(ship->target);

  vx = v * cos(angle * ANGLE_RATIO);
  vy = v * sin(angle * ANGLE_RATIO);

  AnimatedShot::calculate();
}

void VuxLimpet::inflict_damage(SpaceObject *other)
{
  if(other->getID() != SPACE_SHIP)
    return;

  if(!ship)
    return;

  stop_sample(ship->data->sampleSpecial[1]);
  play_sample(ship->data->sampleSpecial[1], 255, 128, 1000, FALSE);

	//MYCODE begin
	BITMAP *bmp=ship->target->getSprite()->get_bitmap(0);
	double ang,dist;
	int hx,hy,tries=0;

	//find a random spot on the target ship where it "exist" 
	do {
		ang=double(random() % 360);
		dist=double(random() % 100)/100.0;
		hx=int((0.50+cos(ang*ANGLE_RATIO)*0.35*dist)*double(bmp->w) );
		hy=int((0.50+sin(ang*ANGLE_RATIO)*0.35*dist)*double(bmp->h) );
		tries++; // just for robustness... don't try forever.
	} while ( (getpixel(bmp,hx,hy)==bitmap_mask_color(bmp)) && (tries<20) );

  // draw the Limplet on the ship panel
  draw_rle_sprite(
	  ship->target->panelBitmap[0],
    sprite->get_sprite(sprite_index, 1), 
		14 + 15+ int(15.0*cos(ang*ANGLE_RATIO)*dist), 
		16 + 16+ int(16.0*sin(ang*ANGLE_RATIO)*dist) 
	);
  ship->target->update_panel = TRUE;

  // draw the Limplet on the ship!
	for (int i=0; i<64; i++) {
		draw_rle_sprite(ship->target->getSprite()->get_bitmap(i),
			sprite->get_sprite(sprite_index, 1), 
			int((0.50+cos((i*360.0/64.0+ang)*ANGLE_RATIO)*0.35*dist)*double(bmp->w) )-4,  
			int((0.50+sin((i*360.0/64.0+ang)*ANGLE_RATIO)*0.35*dist)*double(bmp->h) )-4
		);
	}
	//MYCODE end

  other->speed_loss = slowdown_factor;
  other->handle_damage();
  state = 0;
}

Ship *getVuxIntruder(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData) {
  return(new VuxIntruder(sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO VuxIntruderInfo =
{
  "vuxin",
  "Vux Intruder",
  getVuxIntruder
};
