
#include <math.h>
#include "../melee.h"

class KzerZaDreadnought : public Ship {
  double weaponRange;
  double weaponVelocity;
  int    weaponDamage;
  int    weaponArmour;

  int    specialPassiveFrames;
  int    specialAggresiveFrames;
  int    specialLaserDamage;
  int    specialLaserColor;
  double specialLaserRange;
  int    specialLaserFrames;
  int    specialLaserDrain;
  double specialVelocity;
  double specialRange;
  int    specialArmour;

  public:
  KzerZaDreadnought(double ox, double oy, double shipAngle,
    int shipCollideFlag, int shipRadarColor, ShipData *shipData);

  int activate_weapon();
  int activate_special();
};

class KzerZaMissile : public Missile {
  public:
  KzerZaMissile(double ox, double oy, double oangle, double ov, int odamage,
    double orange, int oarmour, Ship *oship, SpaceSprite *osprite);
};

class KzerZaFighter : public Missile {
  int    passive_frames;
  int    aggresive_frames;
  int    laser_damage;
  int    laser_color;
  double laser_range;
  int    laser_frames;
  int    batt;
  int    recharge_frames;

  public:
  KzerZaFighter(double ox, double oy, double oangle, double ov,
    double orange, int oarmour, int opassive_frames, int oaggresive_frames,
    int olaser_damage, int olaser_color, double olaser_range,
    int olaser_frames, int odrain, Ship *oship, SpaceSprite *osprite);

  void calculate();
  void inflict_damage(SpaceObject *other);
};

KzerZaDreadnought::KzerZaDreadnought(double ox, double oy, double shipAngle,
  int shipRadarColor, int shipCollideFlag, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipRadarColor, shipCollideFlag, shipData) {
  set_config_data((char *)(data->data[0].dat), data->data[0].size);
  weaponRange         = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponVelocity      = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponDamage        = get_config_int("Weapon", "Damage", 0);
  weaponArmour        = get_config_int("Weapon", "Armour", 0);

  specialPassiveFrames   = get_config_int("Special", "PassiveFrames", 0);
  specialAggresiveFrames = get_config_int("Special", "AggresiveFrames", 0);
  specialLaserDamage     = get_config_int("Special", "LaserDamage", 0);
  specialLaserColor      = get_config_int("Special", "LaserColor", 0);
  specialLaserRange      = (double)(get_config_int("Special", "LaserRange", 0)) * RANGE_RATIO;
  specialLaserFrames     = get_config_int("Special", "LaserFrames", 0);
  specialLaserDrain      = get_config_int("Special", "LaserDrain", 0);
  specialVelocity        = (double)(get_config_int("Special", "Velocity", 0)) / VELOCITY_RATIO;
  specialRange           = (double)(get_config_int("Special", "Range", 0)) * RANGE_RATIO;
  specialArmour          = get_config_int("Special", "Armour", 0);
}

int KzerZaDreadnought::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new KzerZaMissile(
    0.0, -(height() / 2.0), angle, weaponVelocity, weaponDamage, weaponRange,
    weaponArmour, this, data->spriteWeapon));
  return(TRUE);
}

int KzerZaDreadnought::activate_special()
{
  if(crew > 1) {
    spaceFrame->addSpaceObject(LAYER_SHOTS, new KzerZaFighter(-25.0, 50.0,
      angle - 157.5, specialVelocity, specialRange, specialArmour,
      specialPassiveFrames, specialAggresiveFrames, specialLaserDamage,
      specialLaserColor, specialLaserRange, specialLaserFrames,
      specialLaserDrain, this, data->spriteSpecial));
    crew--;
    if(crew > 1) {
      spaceFrame->addSpaceObject(LAYER_SHOTS, new KzerZaFighter(25.0, 50.0,
        angle + 157.5, specialVelocity, specialRange, specialArmour,
        specialPassiveFrames, specialAggresiveFrames, specialLaserDamage,
        specialLaserColor, specialLaserRange, specialLaserFrames,
        specialLaserDrain, this, data->spriteSpecial));
      crew--;
    }
    return(TRUE);
  } else
    return(FALSE);
}

KzerZaMissile::KzerZaMissile(double ox, double oy, double oangle, double ov,
  int odamage, double orange, int oarmour, Ship *oship, SpaceSprite *osprite) :
  Missile(ox, oy, oangle, ov, odamage, orange, oarmour, oship, oship,
    osprite) {
  explosionSprite     = ship->data->spriteWeaponExplosion;
  explosionFrameCount = 20;
  explosionFrameSize  = 5;
}

KzerZaFighter::KzerZaFighter(double ox, double oy, double oangle, double ov,
  double orange, int oarmour, int opassive_frames, int oaggresive_frames,
  int olaser_damage, int olaser_color, double olaser_range, int olaser_frames,
  int odrain, Ship *oship, SpaceSprite *osprite) :
  Missile(ox, oy, oangle, ov, 1, orange, oarmour, oship, oship, osprite),
  passive_frames(opassive_frames),
  aggresive_frames(oaggresive_frames),
  laser_damage(olaser_damage),
  laser_color(olaser_color),
  laser_range(olaser_range),
  laser_frames(olaser_frames),
  batt(0),
  recharge_frames(odrain)
{
}

void KzerZaFighter::calculate()
{
  Missile::calculate();

  if(passive_frames > 0) {
    passive_frames-= 1;
    return;
  }

  if(aggresive_frames > 0)
    aggresive_frames-= 1;

  if((ship->target == NULL) || (aggresive_frames == 0) || ship->target->isAlly(ship)) {
    collide_flag = collide_flag | 0x0100;
    angle = trajectory_angle(ship);
    vx = v * cos(angle * ANGLE_RATIO);
    vy = v * sin(angle * ANGLE_RATIO);
    sprite_index = (int)(angle / 5.625) + 16;
    while(sprite_index < 0)
      sprite_index += 64;
    while(sprite_index >= 64)
      sprite_index -= 64;
    return;
  }

  if((distance(ship->target) < laser_range) && !ship->target->isAlly(ship)) {
    collide_flag = collide_flag & 0x0ff;
    vx = 0.0;
    vy = 0.0;
    if(batt == 0) {
      stop_sample(ship->data->sampleExtra[0]);
      play_sample(ship->data->sampleExtra[0], 255, 128, 1000, FALSE);
      spaceFrame->addSpaceLine(new Laser(trajectory_angle(ship->target),
        pallete_color[laser_color], laser_range, laser_damage,
        laser_frames, this, 0.0, -(height() / 2.0)));
      batt = recharge_frames;
    } else
      batt--;
  } else {
    angle = trajectory_angle(ship->target);
    vx = v * cos(angle * ANGLE_RATIO);
    vy = v * sin(angle * ANGLE_RATIO);
    sprite_index = (int)(angle / 5.625) + 16;
    while(sprite_index < 0)
      sprite_index += 64;
    while(sprite_index >= 64)
      sprite_index -= 64;
  }
}

void KzerZaFighter::inflict_damage(SpaceObject *other)
{
  if(other == ship) {
    stop_sample(ship->data->sampleExtra[1]);
    play_sample(ship->data->sampleExtra[1], 255, 128, 1000, FALSE);
    ship->repair++;
    ship->handle_damage();
    state = 0;
  } else {
    state = 0;
  }
}

Ship *getKzerZaDreadnought(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new KzerZaDreadnought(
    sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}

SHIP_REGISTER_INFO KzerZaDreadnoughtInfo =
{
  "kzedr",
  "Kzer-Za Dreadnought",
  getKzerZaDreadnought
};
