
#include <math.h>
#include "../melee.h"

class MiGoMirage : public Ship {
  int    weaponFrames;
  int    weaponDamage;
  double       weaponVelocity;
  int          weaponArmour;

  double       specialForce;
  SpaceSprite *specialSprite;

  double extraRange;
  int    extraDamage;
  int    extraFrames;
  int    extraRechargeRate;
  int    extraColor;
  int    extraArmour;

  public:

  double weaponRange;
  int numParts;
  int whichParts;

  MiGoMirage(double ox, double oy, double shipAngle, int shipCollideFlag,
    int shipRadarColor, ShipData *shipData);

  virtual int activate_weapon();
  virtual void calculate_fire_special();
  virtual void materialize();

  virtual void calculate();

  int addPart(int position);
};

#define part_range 1.25
#define part_speed 1.25
#define part_turn  1.41422

class MiGoMiragePart : public SpaceObject {
  int frame;
  MiGoMirage *ship;

  int whichPart;

  double lRange;
  int    lDamage;
  int    lFrames;
  int    lRechargeRate;
  int    lRecharge;
  int    lColor;
  int    armour;

  public:
  MiGoMiragePart(double oangle, double orange, int odamage, int oframes,
    int orechargerate, int ocolor, int oarmour, MiGoMirage *oship,
    SpaceSprite *osprite, int PartNumber);

  virtual int canCollide(SpaceLocation *other);

  virtual void calculate();
  virtual void handle_damage();
};

MiGoMirage::MiGoMirage(double ox, double oy, double shipAngle,
  int shipCollideFlag, int shipRadarColor, ShipData *shipData) :
  Ship(ox, oy, shipAngle, shipCollideFlag, shipRadarColor, shipData) {
  weaponRange  = (double)(get_config_int("Weapon", "Range", 0)) * RANGE_RATIO;
  weaponFrames = get_config_int("Weapon", "Frames", 0);
  weaponDamage = get_config_int("Weapon", "Damage", 0);
  weaponVelocity = (double)(get_config_int("Weapon", "Velocity", 0)) / VELOCITY_RATIO;
  weaponArmour   = get_config_int("Weapon", "Armour", 0);

  specialForce = (1.0 / (double)(get_config_int("Special", "Force", 1))) / VELOCITY_RATIO;

  extraRange        = (double)(get_config_int("Extra", "Range", 0)) * RANGE_RATIO;
  extraFrames       = get_config_int("Extra", "Frames", 0);
  extraDamage       = get_config_int("Extra", "Damage", 0);
  extraRechargeRate = get_config_int("Extra", "RechargeRate", 0);
  extraColor        = get_config_int("Extra", "Color", 0);
  extraArmour       = get_config_int("Extra", "Armour", 0);

  scale_frame_rate(&extraRechargeRate);

  numParts = 0;
  whichParts = 0;
}

int MiGoMirage::activate_weapon()
{
  spaceFrame->addSpaceObject(LAYER_SHOTS, new Missile(0.0, -(height() / 2.0),
    angle, weaponVelocity, weaponDamage, weaponRange, weaponArmour,
    this, this, data->spriteShip));
  return(TRUE);
}
void MiGoMirage::calculate_fire_special()
{
}

int MiGoMirage::addPart(int position) {
  if ((position < 0) || (position >= 6)) return false;
  if (whichParts & (1<<position)) return false;

  numParts += 1;
  whichParts |= (1<<position);

  spaceFrame->addSpaceObject(LAYER_SHIPS, new MiGoMiragePart(30.0 + 60*position + angle, extraRange,
    extraDamage, extraFrames, extraRechargeRate, extraColor, extraArmour, this,
    data->spriteShip, position));

  weaponRange *= part_range;
  weapon_drain += 1;
//  mass += 3;
  speed_max /= part_speed;
  turn_rate /= part_turn;
  return true;
}
void MiGoMirage::materialize()
{
addPart(0);
addPart(1);
addPart(2);
addPart(3);
addPart(4);
addPart(5);
}

void MiGoMirage::calculate()
{
  if(fire_special) {
    SpaceObject *o;

    for(int i = 0; (i < SPACE_LAYERS); i++) {
      o = spaceFrame->getFirstObject(i);
      while(o != NULL) {
        if((o->getID() == SPACE_ASTEROID) && (distance(o) < 1 * RANGE_RATIO)) {
          damage_factor = 1;
          inflict_damage(o);
          damage_factor = 0;
          batt = batt_max;
          if (addPart(random()%6)) crew += 6;
          else if (addPart(random()%6)) crew += 3;

        }
        o = o->next;
      }
    }

  }
  Ship::calculate();
}

MiGoMiragePart::MiGoMiragePart(double oangle, double orange, int odamage,
  int oframes, int orechargerate, int ocolor, int oarmour, MiGoMirage *oship,
  SpaceSprite *osprite, int PartNumber) :
  SpaceObject(0.0, 0.0, 0.0, oship->collideFlag(), osprite),
  ship(oship),
  whichPart(PartNumber),
  lRange(orange),
  lDamage(odamage),
  lFrames(oframes),
  lRechargeRate(orechargerate),
  lRecharge(0),
  lColor(ocolor),
  armour(oarmour)
{
  angle = oangle;
  x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
  y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);

  if(!ship->exists())
    state = 0;
}

int MiGoMiragePart::canCollide(SpaceLocation *other)
{
  if(!SpaceObject::canCollide(other))
    return(FALSE);

  if(((other->getID() & SPACE_LINE) == 0) && (other->getID() != SPACE_SHOT))
    return(FALSE);

  return(TRUE);
}

void MiGoMiragePart::calculate()
{
  int i;
  SpaceObject *o;

  if(lRecharge > 0)
    lRecharge--;

  SpaceObject::calculate();

  if(!ship->exists()) {
    state = 0;
    return;
  }

  x = ship->normal_x() + (cos(angle * ANGLE_RATIO) * 100.0);
  y = ship->normal_y() + (sin(angle * ANGLE_RATIO) * 100.0);
  angle += ship->turn_rate * (ship->turn_right-ship->turn_left);
  if(angle >= 360.0) angle -= 360.0;
  if(angle < 0) angle += 360.0;
//  sprite_index = ship->sprite_index;

  if (ship->fire_special) return;

  for(i = 0; (lRecharge == 0) && (i < SPACE_LAYERS) && (ship->batt >= ship->special_drain); i++) {
    o = spaceFrame->getFirstObject(i);
    while((lRecharge == 0) && (o != NULL)) {
      if((!o->isAlly(this)) && (o->collideFlag() >= 0) &&
        (distance(o) < lRange)) {
        spaceFrame->addSpaceLine(new Laser(trajectory_angle(o),
          pallete_color[lColor], lRange, lDamage, lFrames, this, 0.0, 0.0));
        lRecharge = lRechargeRate;
        ship->batt -= ship->special_drain;
      }
      o = o->next;
    }
  }
}

void MiGoMiragePart::handle_damage() {
  int d;
  if((damage > 0) || (directDamage > 0)) {
    d = damage + directDamage;
    if (armour > d) {
      ship->damage += d;
      ship->handle_damage();
      }
    else {
      ship->damage += armour;
      ship->handle_damage();
      }
    armour -= d;
    damage = 0;
    directDamage = 0;
    if(armour <= 0) {

      ship->weaponRange /= part_range;
      ship->weapon_drain -= 1;
//      ship->mass -= 3;

      ship->speed_max *= part_speed;
      ship->turn_rate *= part_turn;

      ship->numParts -= 1;
      ship->whichParts &= ~(1<<whichPart);


      armour = 0;
      state = 0;
      spaceFrame->addSpaceObject(LAYER_EXPLOSIONS, new Animation(x, y,
        kaboomSprite, KABOOM_FRAMES, 5));
      stop_sample(ship->data->sampleExtra[0]);
      play_sample(ship->data->sampleExtra[0], 255, 128, 1000, FALSE);
    }
  }
}

Ship *getMiGoMirage(double sX, double sY, double sAngle,
  int sCollideFlag, int sRadarColor, ShipData *sData)
{
  return(new MiGoMirage(sX, sY, sAngle, sCollideFlag, sRadarColor, sData));
}


SHIP_REGISTER_INFO MiGoMirageInfo =
{
  "migmi",
  "Mi-Go Mirage",
  getMiGoMirage
};
